/**********************************************************************************************************************
 * \file Cpu0_Main.c
 * \copyright Copyright (C) Infineon Technologies AG 2019
 * 
 * Use of this file is subject to the terms of use agreed between (i) you or the company in which ordinary course of 
 * business you are acting and (ii) Infineon Technologies AG or its licensees. If and as long as no such terms of use
 * are agreed, use of this file is subject to following:
 * 
 * Boost Software License - Version 1.0 - August 17th, 2003
 * 
 * Permission is hereby granted, free of charge, to any person or organization obtaining a copy of the software and 
 * accompanying documentation covered by this license (the "Software") to use, reproduce, display, distribute, execute,
 * and transmit the Software, and to prepare derivative works of the Software, and to permit third-parties to whom the
 * Software is furnished to do so, all subject to the following:
 * 
 * The copyright notices in the Software and this entire statement, including the above license grant, this restriction
 * and the following disclaimer, must be included in all copies of the Software, in whole or in part, and all 
 * derivative works of the Software, unless such copies or derivative works are solely in the form of 
 * machine-executable object code generated by a source language processor.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE 
 * COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN 
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 *********************************************************************************************************************/
 /*\title Interrupt generation via STM trigger
 * \abstract The STM is configured to trigger an interrupt every 500 ms. The interrupt toggles an LED.
 * \description The System Timer (STM) module counts up to a configured compare value, when it reaches the specific
 *              value, it triggers an interrupt and resets its counter value. The value is set to trigger the interrupt
 *              every 500 ms; at each interrupt an LED is toggled.
 *
 * \name STM_Interrupt_1_KIT_TC277_TFT
 * \version V1.0.2
 * \board APPLICATION KIT TC2X7 V1.1, KIT_AURIX_TC277_TFT_DC-Step, TC27xTP_D-Step
 * \keywords AURIX, Interrupt, LED, STM, STM_Interrupt_1, blink
 * \documents https://www.infineon.com/aurix-expert-training/Infineon-AURIX_STM_Interrupt_1_KIT_TC277_TFT-TR-v01_00_02-EN.pdf
 * \documents https://www.infineon.com/aurix-expert-training/TC27D_iLLD_UM_1_0_1_12_0.chm
 * \lastUpdated 2020-12-18
 *********************************************************************************************************************/
#include "Ifx_Types.h"
#include "IfxCpu.h"
#include "IfxScuWdt.h"

#include "STM.h"
#include "GPT12.h"
#include "TLE9180.h"
#include "Encoder.h"
#include "Current.h"

#include "FOCConfig.h"
#include "TorqueControl.h"
#include "VelocityControl.h"
#include "VADC.h"
#include "PWM.h"

VelocityControl g_velocityControl;

IfxCpu_syncEvent g_cpuSyncEvent = 0;

uint32 Task1ms;
uint32 Task200ms;

uint16 Call_Loop = 0;
float32 velocity;
uint16 Number_of_Fails;
uint8 x = 0;

int core0_main(void)
{
    IfxCpu_enableInterrupts();
    
    /* !!WATCHDOG0 AND SAFETY WATCHDOG ARE DISABLED HERE!!
     * Enable the watchdogs and service them periodically if it is required
     */
    IfxScuWdt_disableCpuWatchdog(IfxScuWdt_getCpuWatchdogPassword());
    IfxScuWdt_disableSafetyWatchdog(IfxScuWdt_getSafetyWatchdogPassword());
    
    IfxCpu_disableInterrupts();

    PWM_Init();
    ENC_init();

    STM_init();
    TLE9180_init();
    AppVadc_init();

    TQC_initControl(&g_motor, 20000);

	VelocityControl_Init(&g_velocityControl);
	VelocityControl_SetKpKi(&g_velocityControl, VELOCITY_CONTROL_KP, VELOCITY_CONTROL_KI, VELOCITY_CONTROL_PERIOD,0, 0);
	VelocityControl_SetLimit(&g_velocityControl, VELOCITY_CONTROL_MIN, VELOCITY_CONTROL_MAX);
	VelocityControl_SetMaxSpeed(&g_velocityControl, VELOCITY_CONTROL_MAX_RPM);
	VelocityControl_SetRefSpeed(&g_velocityControl, VELOCITY_CONTROL_REF);

    TLE9180_loadStartupConfiguration();
	VelocityControl_Enable(&g_velocityControl);

	Task1ms = STM_getTick();
	Task200ms = STM_getTick();

	IfxCpu_enableInterrupts();
	TLE9180_readRegister();
    /* Wait for CPU sync event */
    IfxCpu_emitEvent(&g_cpuSyncEvent);
    IfxCpu_waitEvent(&g_cpuSyncEvent, 1);

    while(1) {
    	if(STM_getElapsedTick(Task1ms) > 1) {

    		g_velocityControl.speed = IfxStdIf_Pos_radsToRpm(ENC_getSpeed());
			// Control Speed if enabled
			if (VelocityControl_IsEnabled(&g_velocityControl)) {// set reference Speed

				FOC_SetIq(&g_motor.control.foc,
						VelocityControl_Do(&g_velocityControl,
								g_velocityControl.speed)
								* FOC_GetIqLimit(&g_motor.control.foc));

			}
    		Task1ms = STM_getTick();
    	}

    	if(STM_getElapsedTick(Task200ms) > 200) {

#if (Sensor == Encoder)

			if ((g_motor.control.calon == TRUE)
					& (g_motor.diag.sync_topzero == FALSE)) //Calibration
					{
				Call_Loop++;

				if (Call_Loop == 1) {
					g_motor.diag.focControlEnabled = FALSE;
					g_motor.diag.encoderEnabled = FALSE;
					IfxStdIf_Pos_setOffset(&StdIfPosHandler, 0);
					g_motor.openLoop.amplitude = 0;
					g_motor.openLoop.electricalAngleDelta = 0;
					g_motor.openLoop.electricalAngle = 0;
					g_motor.openLoop.amplitude = 0.05;
					g_motor.control.start = 1;
				}

				if (Call_Loop == 2) {
					g_motor.openLoop.amplitude = 0.1;
				}
				if (Call_Loop == 5) {
					g_motor.openLoop.amplitude = 0.15;
				}
				if (Call_Loop == 10) {
					//IFX_INLINE void PosIf_setOffset_base(PosIf *handle, PosIf_Raw offset);
					IfxStdIf_Pos_setOffset(&StdIfPosHandler, GPT12Handler.rawPosition);
					g_motor.control.calon = FALSE;
					g_motor.openLoop.amplitude = 0.0;
					g_motor.control.stop = TRUE;
					g_motor.diag.focControlEnabled = TRUE;
					g_motor.diag.encoderEnabled = TRUE;
					Call_Loop = 0;
				}
			}
#endif
			//*************************************************************************************************************
			if ((g_motor.control.run == TRUE)
					& (g_motor.control.calon == FALSE)) {
#if (Sensor == Encoder)
				if (g_motor.diag.sync_topzero == TRUE) //Top Zero Finder
				{

					g_motor.openLoop.amplitude = amplitude_openLoop;
					g_motor.openLoop.electricalAngleDelta = speed_openLoop;
					x++;
				}

				if (x == got_topzero) {
					g_motor.diag.sync_topzero = FALSE;
					g_motor.openLoop.amplitude = 0.0;
					g_motor.openLoop.electricalAngleDelta = 0;
					g_motor.diag.focControlEnabled = TRUE;
					g_motor.diag.encoderEnabled = TRUE;
					g_motor.control.stop = TRUE;
					x = 0;
				}
#endif
			}
			//******************************************************************************************************************
			if ((g_motor.control.run == TRUE) & (g_velocityControl.ref > 0.0)
					& (g_motor.diag.focControlEnabled == TRUE)) // evaluate stall condition of the motor
					{
				if (g_velocityControl.speed > min_speed) {
					Number_of_Fails = 0;
				} else {
					Number_of_Fails++;
					if (Number_of_Fails > fails) // stall condition or motor runs out of control
							{
						g_motor.control.stop = TRUE;
//						g_motor.control.hadEmergency = TRUE;
						g_motor.control.demorun = 0;
					}
				}

				if (((g_velocityControl.speed - g_velocityControl.ref)
						> delta_speed)
						| (g_velocityControl.speed > VELOCITY_CONTROL_MAX_RPM)) //motor runs out of control
						{
					g_motor.control.stop = TRUE;
//					g_motor.control.hadEmergency = TRUE;
					g_motor.control.demorun = 0;
				}

			}
			//********************************************************************************************************************

			if (g_motor.control.demo == 1) {
				g_motor.control.start = 1;
				g_motor.control.demorun = 1;
				velocity = VelocityControl_GetRefSpeed(&g_velocityControl);
				velocity = velocity + 50.0;
				VelocityControl_SetRefSpeed(&g_velocityControl, velocity);

				if (velocity > (VELOCITY_CONTROL_MAX_RPM - 3500))

				{

					g_motor.control.demorun = 0;
					g_motor.control.demo = 0;
					g_motor.control.demostop = 1;

				}
			}

			if (g_motor.control.demostop == 1) {
				velocity = VelocityControl_GetRefSpeed(&g_velocityControl);
				velocity = velocity - 50.0;
				VelocityControl_SetRefSpeed(&g_velocityControl, velocity);

				if (velocity <= 0) {
					g_motor.control.start = 0;
					g_motor.control.demostop = 0;
					g_motor.control.run = 0;
					velocity = 0;
					VelocityControl_SetRefSpeed(&g_velocityControl, velocity);
				}
			}

			Task200ms = STM_getTick();
    	}
    }
    return (1);
}
