/*
 * FOC.c
 *
 *  Created on: 2021. 3. 11.
 *      Author: User
 */

#include "FOC.h"
#include "Tables.h"
#include "math.h"

IFX_INLINE void FOC_InitComplex(cfloat32* compl)
{
    compl->imag = 0.0;
    compl->real = 0.0;
}


IFX_INLINE cfloat32 FOC_LimitVqVd(TPic* PIC_d,TPic* PIC_q, cfloat32 mdq,float32 Limit )
{
    /* Note 0.9 is the maximum for where Vref in the lies in the circle (continuous) */
	float32 ModIndex = sqrt((mdq.imag * mdq.imag) + (mdq.real * mdq.real));
    ModIndex = (Limit / ModIndex );
    if(ModIndex < 1)
    {
    	mdq.imag *= ModIndex;
    	mdq.real *= ModIndex;
    	// Correct PI-Output with new calculated values for anti wind up reason
    	PIC_d->uk = mdq.real;
    	PIC_q->uk = mdq.imag;
    }
    return mdq;
}

/** @brief Initialize the Foc object.
 *
 * This function initializes the Foc object:
 *    - PiId and PiIq parameters are set to default values as done by PIC_Init()
 *    - The Id and Iq references and limits are set to 0
 *
 * @param Foc Specifies the field oriented control object.
 *
 * @return none
 * @note The Kp, Ki and limits of the PI controllers must be set separately.
 * @see PIC_Init(), PIC_SetKpKi(), PIC_SetKpKi_float32(), PIC_SetLimit(), FOC_SetIdLimit(), FOC_SetIqLimit(), FOC_SetId(), FOC_SetIq().
 * @ingroup motor_control_foc
 */
boolean FOC_Init(TFoc *foc)
{
    FOC_InitComplex(&foc->ref);
    FOC_InitComplex(&foc->limit);
    PIC_Init(&foc->piId);
    PIC_Init(&foc->piIq);

    return TRUE;
}

IFX_INLINE void FOC_SetPiLimit(TPic* pic, float32 orthoRef)
{
    /* Note 0.9 is the maximum for where Vref in the lies in the circle (continuous) */
    float32 limit = 0;//sqrt_f((0.9 * 0.9) - (orthoRef * orthoRef));
    PIC_SetLimit(pic, -limit, limit);
}


/** @brief Field Oriented Control function.
 *
 * This function performs field oriented control calculations, and returns the output of the reverse parke transformation.
 *
 * @param Foc Specifies the field oriented control object.
 * @param ElectricalAngle Specifies motor electrical angle between 0 an 2*PI where 2*PI corresponds to COSSIN_TABLE_SIZE.
 * @param Currents Specifies the 3 phases currents in the (a, b, c) frame. which are the input for the Clarke transformation.
 *
 * @return mab
 * @see Clarke(), Park(), Park_Reverse(), Pic().
 * @ingroup motor_control_foc
 */

float32 FOC_Limit = 0.85; // pi/(2* sqrt(3)) + 2*DT/Periode

cfloat32 FOC_Do(TFoc *foc, sint32 electricalAngle, T3Phase *currents)
{
	cfloat32 cosSin;
	cfloat32 mdq, mab;

    /* Clarke Park transformation */
    mab = TRSF_clark(currents);

    cosSin = TBL_cosinusSinus(electricalAngle);

    mdq = TRSF_park(&mab, &cosSin);

    foc->nfo_mdq = mdq;

    /* Pure PI controller only with symmetrical Limitation of Vq and Vd*/

    /* Pi controller */
    mdq.imag =  PIC_Do(&foc->piIq,(foc->ref.imag - mdq.imag));
    mdq.real =  PIC_Do(&foc->piId, (foc->ref.real - mdq.real));
    // Limitation of Vq and Vd with anti wind up
    mdq = FOC_LimitVqVd(&foc->piId,&foc->piIq, mdq, foc->limitAmpl);

    /* Reverse Park transformation */
    mab = TRSF_parkReverse(&mdq, &cosSin);
    foc->nfo_mab = mab;
    return mab;
}

/** @brief Set the Iq current reference.
 *
 * This function sets the Iq current reference. The Iq reference value is checked against the Iq Limit.
 *
 * @param Foc Specifies the field oriented control object.
 * @param Iq Specifies Iq current reference.
 *
 * @return none
 * @see FOC_GetIq(), FOC_GetIqLimit(), FOC_SetIqLimit().
 * @ingroup motor_control_foc
 */
void FOC_SetIq(TFoc *foc, float32 iq)
{
    foc->ref.imag = __saturate_stdreal(iq, -foc->limit.imag, foc->limit.imag);
}

/** @brief Return the Iq current reference.
 *
 * This function returns the Iq current reference.
 *
 * @param Foc Specifies the field oriented control object.
 *
 * @return Returns the Iq current reference
 * @see FOC_SetIq().
 * @ingroup motor_control_foc
 */
float32 FOC_GetIq(TFoc *foc)
{
    return foc->ref.imag;
}

/** @brief Set the Id current reference.
 *
 * This function sets the Id current reference. The Id reference value is checked against the Id Limit.
 *
 * @param Foc Specifies the field oriented control object.
 * @param Id Specifies Id current reference.
 *
 * @return none
 * @see FOC_GetId(), FOC_GetIdLimit(), FOC_SetIdLimit().
 * @ingroup motor_control_foc
 */
void FOC_SetId(TFoc *foc, float32 id)
{
    foc->ref.real = __saturate_stdreal(id, -foc->limit.real, foc->limit.real);
}

/** @brief Return the Id current reference.
 *
 * This function returns the Id current reference.
 *
 * @param Foc Specifies the field oriented control object.
 *
 * @return Returns the Id current reference
 * @see FOC_SetId().
 * @ingroup motor_control_foc
 */
float32 FOC_GetId(TFoc *foc)
{
    return foc->ref.real;
}

/** @brief Set the Iq current limit.
 *
 * This function sets the Iq current limit.
 *
 * @param Foc Specifies the field oriented control object.
 * @param Limit Specifies Iq current limit.
 *
 * @return none
 * @see FOC_GetIqLimit().
 * @ingroup motor_control_foc
 */
void FOC_SetIqLimit(TFoc *foc, float32 limit)
{
    foc->limit.imag = limit;
}

/** @brief Return the Iq current limit.
 *
 * This function returns the Iq current limit.
 *
 * @param Foc Specifies the field oriented control object.
 *
 * @return Returns the Iq current limit
 * @see FOC_SetIqLimit().
 * @ingroup motor_control_foc
 */
float32 FOC_GetIqLimit(TFoc *foc)
{
    return foc->limit.imag;
}

/** @brief Set the Id current limit.
 *
 * This function sets the Id current limit.
 *
 * @param Foc Specifies the field oriented control object.
 * @param Limit Specifies Id current limit.
 *
 * @return none
 * @see FOC_GetIdLimit().
 * @ingroup motor_control_foc
 */
void FOC_SetIdLimit(TFoc *foc, float32 limit)
{
    foc->limit.real = limit;
}

/** @brief Return the Id current limit.
 *
 * This function returns the Id current limit.
 *
 * @param Foc Specifies the field oriented control object.
 *
 * @return Returns the Id current limit
 * @see FOC_SetIdLimit().
 * @ingroup motor_control_foc
 */
float32 FOC_GetIdLimit(TFoc *foc)
{
    return foc->limit.real;
}

/** @brief Set the Modulation Index limit.
 *
 * This function sets the limit of Modulation Index.
 *
 * @param Foc Specifies the field oriented control object.
 * @param Limit Specifies Modulation Index limit.
 *
 * @return none
 * @see FOC_GetModIndexLimit().
 * @ingroup motor_control_foc
 */
void FOC_SetModIndexLimit(TFoc *foc, float32 limit)
{
    foc->limitAmpl = limit;
}

/** @brief Return the Modulation Index limit.
 *
 * This function returns the limit of Modulation Index.
 *
 * @param Foc Specifies the field oriented control object.
 *
 * @return Returns the Modulation Index limit
 * @see FOC_SetModIndexLimit().
 * @ingroup motor_control_foc
 */
float32 FOC_GetModIndexLimit(TFoc *foc)
{
    return foc->limitAmpl;
}

/** @brief Reset the FOC PI controllers.
 *
 * The PiId and PiIq parameters are set to default values by PIC_Reset()
 *
 * @param Foc Specifies the field oriented control object.
 *
 * @return None.
 * @ingroup motor_control_foc
 */
void FOC_Reset(TFoc *foc)
{
    PIC_Reset(&foc->piId);
    PIC_Reset(&foc->piIq);
}
