/*
 * VelocityControl.h
 *
 *  Created on: 2021. 3. 11.
 *      Author: User
 */

#ifndef MOTOR_CONTROL_VELOCITYCONTROL_H_
#define MOTOR_CONTROL_VELOCITYCONTROL_H_

#include "PI.h"
#include "IfxCpu_Intrinsics.h"
#include "math.h"

#define VELOCITY_kp_multi (0.5)
#define VELOCITY_ki_multi (1.0)


#define VELOCITY_CONTROL_KP			(VELOCITY_kp_multi*(FOC_CALC_KP))    	/**< @brief Define the Kp parameter value. */
#define VELOCITY_CONTROL_KI			(VELOCITY_ki_multi*(FOC_CALC_KI))			/**< @brief Define the Ki parameter value. */
#define VELOCITY_CONTROL_MIN		(0.0)								/**< @brief Define the Pi Controller Out minimum value. */
#define VELOCITY_CONTROL_MAX		(1.0)								/**< @brief Define the Pi Controller Out maximum value. */
#define VELOCITY_CONTROL_MAX_RPM	(7000.0)							/**< @brief Define the Max rpm in 1/min value. */
#define VELOCITY_CONTROL_PERIOD		(1e-3)								/**< @brief Define the Controller period in seconds value. */
#define VELOCITY_CONTROL_REF		(0.0)							/**< @brief Define the Motor REF rpm in 1/min value. */

typedef struct
{
    TPic pi;                           /**< @brief Speed PI controller */
    float32 speed;                    /**< @brief Speed acquired from position sensor */
    float32 ref;                      /**< @brief Speed reference in rad/s */
    float32 max;                      /**< @brief Absolute value of the max allowed speed reference in rad/s. Range=[0, +INF] */
    boolean enabled;                      /**< @brief Speed control enable flag. TRUE: the speed control is enabled. FALSE the speed control is disabled */
} VelocityControl;

void VelocityControl_Init(VelocityControl *velocityControl);
float32 VelocityControl_Do(VelocityControl *velocityControl, float32 speed);

/** @brief Reset the velocity controller.
 *
 * @param VelocityControl Specifies the velocity control object.
 *
 * @return none
 * @ingroup app_velocity_control
 */
IFX_INLINE void VelocityControl_Reset(VelocityControl *velocityControl)
{
    velocityControl->ref = 0;
    PIC_Reset(&velocityControl->pi);
}

/** @brief Enable the velocity controller.
 *
 * @param VelocityControl Specifies the velocity control object.
 *
 * @return none
 * @ingroup app_velocity_control
 */
IFX_INLINE void VelocityControl_Enable(VelocityControl *velocityControl)
{
    velocityControl->enabled = TRUE;
}
/** @brief Return the status of the velocity controller (Enabled / Disabled).
 *
 * @param VelocityControl Specifies the velocity control object.
 *
 * @retval TRUE Returns TRUE if the velocity controller is enabled
 * @retval FALSE Returns FALSE if the velocity controller is disabled
 * @ingroup app_velocity_control
 */
IFX_INLINE boolean VelocityControl_IsEnabled(VelocityControl *velocityControl)
{
    return velocityControl->enabled;
}
/** @brief Disable the velocity controller.
 *
 * @param VelocityControl Specifies the velocity control object.
 *
 * @return none
 * @ingroup app_velocity_control
 */
IFX_INLINE void VelocityControl_Disable(VelocityControl *velocityControl)
{
    velocityControl->enabled = FALSE;
    VelocityControl_Reset(velocityControl);
}

/** @brief Reset the velocity controller limit flag.
 *
 * @param VelocityControl Specifies the velocity control object.
 *
 * @return none
 * @ingroup app_velocity_control
 */
IFX_INLINE void VelocityControl_ResetLimitFlag(VelocityControl *velocityControl)
{
    PIC_ResetLimitsHit(&velocityControl->pi);
}

/** @brief Set the velocity controller limits.
 *
 * @param VelocityControl Specifies the velocity control object.
 * @param Min Specifies the PI controller min value.
 * @param Max Specifies the PI controller max value.
 *
 * @return none
 * @see PIC_SetLimit()
 * @ingroup app_velocity_control
 */
IFX_INLINE void VelocityControl_SetLimit(VelocityControl *velocityControl, float32 min, float32 max)
{
    PIC_SetLimit(&velocityControl->pi, min, max);
}
/** @brief Set the velocity controller max ref speed.
 *
 * @param VelocityControl Specifies the velocity control object.
 * @param Max Specifies the velocity controller max ref speed value.
 *
 * @return none
 * @ingroup app_velocity_control
 */
IFX_INLINE void VelocityControl_SetMaxSpeed(VelocityControl *velocityControl, float32 max)
{
    velocityControl->max = max;
}

/** @brief Set the velocity controller reference speed.
 *
 * @param VelocityControl Specifies the velocity control object.
 * @param Speed Specifies the velocity controller reference speed value.
 *
 * @retval TRUE Returns TRUE if the reference speed could be set
 * @retval FALSE Returns FALSE if the reference speed is out of range
 * @ingroup app_velocity_control
 */
IFX_INLINE boolean VelocityControl_SetRefSpeed(VelocityControl *velocityControl, float32 speed)
{
    boolean result;
    if ((__absf(speed)) > (velocityControl->max))
    {
        result = FALSE;
    }
    else
    {
        velocityControl->ref = speed;
        result = TRUE;
    }
    return result;
}

/** @brief Return the velocity controller max ref speed.
 *
 * @param VelocityControl Specifies the velocity control object.
 *
 * @return Return the max allowed ref speed
 * @ingroup app_velocity_control
 */
IFX_INLINE float32 VelocityControl_GetMaxSpeed(VelocityControl *velocityControl)
{
    return velocityControl->max;
}
/** @brief Return the velocity controller ref speed.
 *
 * @param VelocityControl Specifies the velocity control object.
 *
 * @return Return the ref speed
 * @ingroup app_velocity_control
 */
IFX_INLINE float32 VelocityControl_GetRefSpeed(VelocityControl *velocityControl)
{
    return velocityControl->ref;
}
/** @brief Return a copy of the velocity controller PI controller.
 *
 * @param VelocityControl Specifies the velocity control object.
 * @param Pi Specifies the location where the copy must be saved.
 *
 * @return Return the max allowed ref speed
 * @ingroup app_velocity_control
 */
IFX_INLINE void VelocityControl_GetPi(VelocityControl *velocityControl, TPic *pi)
{
    *pi = velocityControl->pi;
}
/** @brief Set the velocity controller PI parameters
 *
 * @param VelocityControl Specifies the velocity control object.
 * @param Kp Specifies the PI controller Kp value.
 * @param Ki Specifies the PI controller Ki value.
 * @param Period Specifies PI controller period time in seconds.
 * @param ShiftCoef position of the fix point for the A0 and A1 parameters. Range = [-256, 255]. In case TStdReal type is float32, this parameter is ignored
 * @param ShiftResult Shift done on the result for fixpoint calculation. Range = [-32, 31]. In case TStdReal type is float32, this parameter is ignored
 *
 * @return None.
 * @see PIC_SetKpKi_StdReal()
 * @ingroup app_velocity_control
 */
IFX_INLINE void VelocityControl_SetKpKi(VelocityControl *velocityControl, float32 kp, float32 ki, float32 period, sint32 shiftCoef, sint8 shiftResult)
{
    PIC_SetKpKi_float32(&velocityControl->pi, kp, ki, period, shiftCoef, shiftResult);
}

IFX_INLINE float32 VelocityControl_GetSpeed(VelocityControl *velocityControl)
{
    return velocityControl->speed;
}


#endif /* MOTOR_CONTROL_VELOCITYCONTROL_H_ */
