/*
 * PI.c
 *
 *  Created on: 2021. 3. 10.
 *      Author: User
 */

#include "PI.h"

/** @brief Initialize the PI controller.
 *
 * This function initializes the PI controller. It must be called before any call to other PI functions.
 *
 * @param Pic Specifies PI controller.
 *
 * @return none

 * @see PIC_SetKpKi(), PIC_SetKpKi_float32(), PIC_SetLimit()
 * @ingroup control_pi
 */
void PIC_Init(TPic * Pic)
{
    Pic->a0 = 0;
    Pic->a1 = 0;
    Pic->ik = 0;
    Pic->uk = 0;
    Pic->min = 0;
    Pic->max = 0;
#if !defined(CFG_OPTIMIZE) || (CFG_OPTIMIZE == 0)
    Pic->hitLimits = FALSE;
#endif
}

/** @brief Set the PI controller Kp and Ki parameter (for fixpoint calculation).
 *
 * This function sets the PI controller Kp and Ki parameter. The A0 and A1 coeficient
 * are converted to fix point values (Qx.y format where x = Shift+1) using the Shift parameter, taking in account the
 * saturation.
 *
 * @param Pic Specifies PI controller.
 * @param Kp Specifies the PI controller Kp value.
 * @param Ki Specifies the PI controller Ki value.
 * @param SampleTime Specifies PI controller sampling time in seconds.
 * @param ShiftCoef position of the fix point for the A0 and A1 parameters. Range = [-256, 255]. In case Tfloat32 type is float32, this parameter is ignored
 * @param ShiftResult Shift done on the result for fixpoint calculation. Range = [-32, 31]. In case Tfloat32 type is float32, this parameter is ignored
 *
 * @return none
 * @see PIC_GetKpKi(), PIC_GetKpKi_float32(), float32_To_FixPoint32()
 * @ingroup control_pi
 */
void PIC_SetKpKi_float32(TPic * pic, float32 kp, float32 ki, float32 sampleTime, sint32 shiftCoef, sint8 shiftResult)
{
    if ((kp < 0) || (kp > 0))
    {
        float32 k = (ki / kp) * (sampleTime / 2);
        pic->a0 = kp * (k - 1);  /* Kp.(Wpi.Tsample/2 - 1); Wpi = Ki/Kp  */
        pic->a1 = kp * (k + 1);  /* Kp.(Wpi.Tsample/2 + 1); Wpi = Ki/Kp  */
    }
    else
    {
        pic->a0 = 0;
        pic->a1 = 0;
    }
}

/** @brief Return the PI controller Kp and Ki parameter.
 *
 * This function returns the PI controller Kp and Ki parameter. The A0 and A1 coeficient
 * are converted to floating point values using the Shift parameter  (Qx.y format where x = Shift+1).
 *
 * @param Pic Specifies PI controller.
 * @param Kp This parameter is filled in by the function with the PI controller Kp value.
 * @param Ki This parameter is filled in by the function with the PI controller Ki value.
 * @param SampleTime Specifies PI controller sampling time in seconds.
 * @param ShiftCoef position of the fix point. Range = [-256, 255]. In case Tfloat32 type is float32, this parameter is ignored
 *
 * @return none
 * @see PIC_SetKpKi(), PIC_SetKpKi_float32(), FixPoint32_To_float32()
 * @ingroup control_pi
 */
void PIC_GetKpKi_float32(TPic * pic, float32 *kp, float32 *ki, float32 sampleTime, sint32 shiftCoef)
{
    *ki = (pic->a1 + pic->a0) / sampleTime;
    *kp = (pic->a1 - pic->a0) / 2;
}

#if STD_REAL_TYPE == STD_REAL_FLOAT_32
/** @brief Set the PI controller Kp and Ki parameter (for Tfloat32 as float32 only).
 *
 * This function sets the PI controller Kp and Ki parameter.
 *
 * @param Pic Specifies PI controller.
 * @param Kp Specifies the PI controller Kp value.
 * @param Ki Specifies the PI controller Ki value.
 * @param SampleTime Specifies PI controller sampling time in seconds.
 *
 * @return none
 * @see PIC_GetKpKi(), PIC_GetKpKi_float32()
 * @ingroup control_pi
 */
void PIC_SetKpKi(TPic * pic, float32 kp, float32 ki, float32 sampleTime)
{
    PIC_SetKpKi_float32(pic, kp, ki, sampleTime, 0, 0);
}

/** @brief Return the PI controller Kp and Ki parameter (for Tfloat32 as float32 only).
 *
 * This function returns the PI controller Kp and Ki parameter.
 *
 * @param Pic Specifies PI controller.
 * @param Kp This parameter is filled in by the function with the PI controller Kp value.
 * @param Ki This parameter is filled in by the function with the PI controller Ki value.
 * @param SampleTime Specifies PI controller sampling time in seconds.
 *
 * @return none
 * @see PIC_SetKpKi(), PIC_SetKpKi_float32(), PIC_GetKpKi_float32()
 * @ingroup control_pi
 */
void PIC_GetKpKi(TPic * pic, float32 *kp, float32 *ki, float32 sampleTime)
{
    PIC_GetKpKi_float32(pic, kp, ki, sampleTime, 0);
}
#endif


/** @brief Set the PI controller limits.
 *
 * This function sets the PI controller limits.
 *
 * @param Pic Specifies PI controller.
 * @param Min Specifies the PI controller min value.
 * @param Max Specifies the PI controller max value.
 *
 * @return none
 * @see PIC_GetLimit()
 * @ingroup control_pi
 */
void PIC_SetLimit(TPic * pic, float32 min,float32 max)
{
    pic->min = min;
    pic->max = max;
#if !defined(CFG_OPTIMIZE) || (CFG_OPTIMIZE == 0)
    pic->hitLimits = FALSE;
#endif
}

/** @brief Return the PI controller limits.
 *
 * This function returns the PI controller limits.
 *
 * @param Pic Specifies PI controller.
 * @param Min This parameter is filled in by the function with the PI controller min value.
 * @param Max This parameter is filled in by the function with the PI controller max value.
 *
 * @return none
 * @ingroup control_pi
 */
void PIC_GetLimit(TPic * pic, float32 *min, float32 *max)
{
    *min = pic->min;
    *max = pic->max;
}


#if !defined(CFG_OPTIMIZE) || (CFG_OPTIMIZE == 0)
/** @brief Test if the PI controller limits have been hit.
 *
 * This function returns TRUE if the PI controller limits have been hit.
 *
 * @param Pic Specifies PI controller.
 *
 * @retval TRUE if the PI controller limits have been hit.
 * @retval FALSE if the PI controller limits have not been hit.
 * @see PIC_SetLimit()
 * @ingroup control_pi
 */
boolean PIC_IsLimitsHit(TPic * pic)
{
    return pic->hitLimits;
}
/** @brief Resets the PI controller limits hit flag.
 *
 * This function resets the PI controller limits hit flag.
 *
 * @param Pic Specifies PI controller.
 *
 * @return None.
 * @see PIC_IsLimitsHit()
 * @ingroup control_pi
 */
void PIC_ResetLimitsHit(TPic * pic)
{
    pic->hitLimits = FALSE;
}

#endif

#if !defined(CFG_OPTIMIZE) || (CFG_OPTIMIZE == 0)
/** @brief Update the PI controller with the new error value.
 *
 * This function updates the PI controller and return the new PI output according to the following formula:
 *  U(k+1)= A1 * I(k+1) + A0 * Ik + Uk
 *
 * In case Fix point calculation is used, the following formula is applyed (saturated shift)
 *  U(k+1)= ((A1 * I(k+1) + A0 * Ik) << ShiftResult) + Uk
 *
 * @param Pic Specifies PI controller.
 * @param Input Specifies the new input error value.
 *
 * @return Returns the new PI output value.
 * @ingroup control_pi
 */
float32 PIC_Do(TPic * pic, float32 input)
{
    float32 uk;
//    Uk = A1 * Input + Uk + A0 * Pic->Ik;
    uk = ((pic->a1 * input) + (pic->a0 * pic->ik)) + pic->uk;

    pic->ik = input;

    if (uk >= pic->max)
    {
        pic->hitLimits = TRUE;
        pic->uk = pic->max;
    }
    else if (uk <= pic->min)
    {
        pic->hitLimits = TRUE;
        pic->uk = pic->min;
    }
    else
    {
        pic->uk = uk;
    }
    return (float32)pic->uk;
}
#endif

/** @brief Reset the PI controller.
 *
 * This function resets the PI controller by setting the Last error and integral part to 0.
 *
 * @param Pic Specifies PI controller.
 *
 * @return none
 * @ingroup control_pi
 */
void PIC_Reset(TPic * pic)
{
    pic->ik = 0;
    pic->uk = 0;
}

